/*============================================================================*/
/*
SimpleRTOS - Very simple RTOS for Microcontrollers
v2.00 (2014-01-21)
isaacbavaresco@yahoo.com.br
*/
/*============================================================================*/
/*

							   PIC32MX210F016B

						+-------------U-------------+
						|							|
					  ==|  1 !MCLR			AVCC 28 |==
						|							|
		  SPI1_SS <-  ==|  2 RA0			AGND 27 |==
						|							|
		 SPI1_SDI  -> ==|  3 RA1			RB15 26 |== <-  SPI2_SCK
						|							|
		 	 PGED  -> ==|  4 RB0			RB14 25 |==  -> SPI1_SCK
						|							|
			 PGEC  -> ==|  5 RB1			RB13 24 |== <-  ANALOG_AN11
						|							|
	   RS232_U1RX  -> ==|  6 RB2		 VUSB3V3 23 |==
	   					|							|
	   RS232_U1TX <-  ==|  7 RB3			RB11 22 |== <-> UBS_D-
						|							|
					  ==|  8 GND			RB10 21 |== <-> UBS_D+
						|							|
					  ==|  9 RA2/OSC1		VCAP 20 |==
						|							|
					  ==| 10 RA3/OSC2		 GND 19 |==
						|							|
			  RST <-  ==| 11 RB4			 RB9 18 |== <-> SPI2_SS/TX/SDA
						|							|
	  SPI2_MISO_2  -> ==| 12 RA4			 RB8 17 |==	<-> SPI2_SDO/RX/SCL
						|							|
					  ==| 13 VCC			 RB7 16 |==  -> IRQ
						|							|
	   	 SPI1_SDO <-  ==| 14 RB5			VBUS 15 |== <-  5V
						|							|
						+---------------------------+


*/
/*============================================================================*/
/*
 Copyright (c) 2007-2014, Isaac Marino Bavaresco
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
     * Neither the name of the author nor the
       names of its contributors may be used to endorse or promote products
       derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ''AS IS'' AND ANY
 EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*============================================================================*/
/*
Set TAB width to 4 characters
*/
/*============================================================================*/
#include <p32xxxx.h>
#include <string.h>

#include "TestApplication.h"
#include "TestPortable.h"
/*============================================================================*/
/* Kernel includes. */
/*============================================================================*/
#include "SimpleRTOS/SimpleRTOS.h"
#include "SimpleRTOS/Queue.h"
#include "SimpleRTOS/Mutex.h"
#include "SimpleRTOS/SerialPIC32.h"
/*============================================================================*/
/** C O N F I G U R A T I O N ************************************************/

#ifndef OVERRIDE_CONFIG_BITS

    #pragma config PMDL1WAY = OFF
    #pragma config IOL1WAY  = OFF
    #pragma config FUSBIDIO = OFF
    #pragma config FVBUSONIO = OFF

    #pragma config FPLLIDIV = DIV_1         // PLL Input Divider
    #pragma config FPLLMUL  = MUL_20        // PLL Multiplier
    #pragma config UPLLIDIV = DIV_1         // USB PLL Input Divider
    #pragma config UPLLEN   = ON            // USB PLL Enabled
    #pragma config FPLLODIV = DIV_2         // PLL Output Divider

    #pragma config FNOSC    = PRIPLL        // Oscillator Selection
    #pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable
    #pragma config POSCMOD  = HS            // Primary Oscillator
    #pragma config OSCIOFNC = ON           // CLKO Enable
    #pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
    #pragma config FCKSM    = CSECME        // Clock Switching & Fail Safe Clock Monitor
    #pragma config FWDTEN   = OFF           // Watchdog Timer

    #pragma config JTAGEN   = OFF
    #pragma config PWP      = OFF           // Program Flash Write Protect
    #pragma config BWP      = OFF           // Boot Flash Write Protect
    #pragma config CP       = OFF           // Code Protect
            
#endif // OVERRIDE_CONFIG_BITS


/** I N C L U D E S **********************************************************/

#include "GenericTypeDefs.h"
#include "USB/usb.h"
#include "USB/usb_function_cdc.h"
#include "USB/Compiler.h"
#include "USB/usb_config.h"
#include "USB/usb_device.h"

#include "HardwareProfile.h"
/*============================================================================*/

#if			defined __XC8__ || defined __18CXX

#pragma config	WDT		= ON,		STVR	= ON,	XINST	= ON
#pragma config	CP0		= ON
#pragma config	FOSC	= HSPLL,	FOSC2	= ON,	FCMEN	= OFF,	IESO	= OFF
#pragma config	WDTPS	= 2048
#pragma config	CCP2MX	= ON,		ECCPMX	= ON,	ETHLED	= OFF
#pragma config	DEBUG	= OFF

#endif	/*	defined __XC8__ || defined __18CXX */


/*============================================================================*/
unsigned char	Stacks[6][APPLICATION_STACK_SIZE];
unsigned char	StackUSB[1024];
context_t		Contexts[7]; /* We will have a maximum of 7 simultaneously running tasks. */
/*  Buffer for USB  */
char USB_In_Buffer[64];
char USB_Out_Buffer[64];
/*============================================================================*/
extern queue_t  QueuesRX[3];
extern queue_t  QueuesTX[3];

/*
mutex_t         MutexUSB;
*/
/*============================================================================*/
static void TaskUSB( void *param )
    {
    USBDeviceInit();    /* usb_device.c.  Initializes USB module SFRs and firmware */
                        /* variables to known states. */
    #if defined(USB_INTERRUPT)
        USBDeviceAttach();
    #endif

    while(1)
        {
        BYTE numBytesRead;
        int i;

        #if defined(USB_POLLING)
        /*  Check bus status and service USB interrupts. */
        USBDeviceTasks();
        #endif

        /*  If suspended, do nothing. */
        while( USBDeviceState < CONFIGURED_STATE || USBSuspendControl == 1 )
            vSleep( 100 );

        numBytesRead = getsUSBUSART( USB_Out_Buffer, sizeof USB_Out_Buffer );
        for( i = 0; i < numBytesRead; i++ )
            QueueWrite( &QueuesRX[2], &USB_Out_Buffer[i], -1 );

        if( mUSBUSARTIsTxTrfReady() )
            {
            for( i = 0; QueueRead( &QueuesTX[2], &USB_In_Buffer[i], 0 ) > 0 && i < sizeof USB_In_Buffer; i++ )
                {}
            if( i > 0 )
                putUSBUSART( USB_In_Buffer, i );
            }

        /*  Service the USB CDC driver */
        CDCTxService();
        }
    }

/*============================================================================*/

PLATFORM_mainreturn_t main( void )
	{
	unsigned int	i;

	for( i = 0; i < sizeof Stacks / sizeof Stacks[0]; i++ )
		memset( Stacks[i], 'A' + i, sizeof Stacks[i] );
    memset( StackUSB, 'U', sizeof StackUSB );

    /* Initialize the contexts and the rest of the RTOS. */
    InitRTOS( Contexts, sizeof Contexts / sizeof Contexts[0] );

	/* Initialise the USB simulated serial before any task try to use it */
    SerialPortInitMinimal( HOST_SERIAL_USB,   115200 );
	/* Create the USB Task */
	CreateTask( StackUSB, sizeof StackUSB, NULL, GetAddress( TaskUSB ), 1 );

    /* Create just one task, it will create the others. */
	CreateTask( Stacks[0], sizeof Stacks[0], NULL, GetAddress( MainTask ), 0 );

    /* From now, the RTOS will take over. */
    StartRTOS( APPLICATION_TICKLENGTH );
   
   	PLATFORM_mainreturn( 0 );
    }
/*============================================================================*/
// ******************************************************************************************************
// ************** USB Callback Functions ****************************************************************
// ******************************************************************************************************
// The USB firmware stack will call the callback functions USBCBxxx() in response to certain USB related
// events.  For example, if the host PC is powering down, it will stop sending out Start of Frame (SOF)
// packets to your device.  In response to this, all USB devices are supposed to decrease their power
// consumption from the USB Vbus to <2.5mA each.  The USB module detects this condition (which according
// to the USB specifications is 3+ms of no bus activity/SOF packets) and then calls the USBCBSuspend()
// function.  You should modify these callback functions to take appropriate actions for each of these
// conditions.  For example, in the USBCBSuspend(), you may wish to add code that will decrease power
// consumption from Vbus to <2.5mA (such as by clock switching, turning off LEDs, putting the
// microcontroller to sleep, etc.).  Then, in the USBCBWakeFromSuspend() function, you may then wish to
// add code that undoes the power saving things done in the USBCBSuspend() function.

// The USBCBSendResume() function is special, in that the USB stack will not automatically call this
// function.  This function is meant to be called from the application firmware instead.  See the
// additional comments near the function.

/******************************************************************************
 * Function:        void USBCBSuspend(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Call back that is invoked when a USB suspend is detected
 *
 * Note:            None
 *****************************************************************************/
void USBCBSuspend( void )
	{
    // Insert appropriate code here for the desired
    // application behavior.  If the microcontroller will be put to sleep, a
    // process similar to that shown below may be used:
       
    /* Example power saving code.  
    
    ConfigureIOPinsForLowPower();
    SaveStateOfAllInterruptEnableBits();
    DisableAllInterruptEnableBits();
    
    // Should enable at least USBActivityIF as a wake source
    EnableOnlyTheInterruptsWhichWillBeUsedToWakeTheMicro();
    Sleep();
    
    // Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.
    RestoreStateOfAllPreviouslySavedInterruptEnableBits();
    
    // Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.
    RestoreIOPinsToNormal();
    
    // IMPORTANT NOTE: Do not clear the USBActivityIF (ACTVIF) bit here.  This bit is 
    // cleared inside the usb_device.c file.  Clearing USBActivityIF here will cause 
    //  to not work as intended.
    */
	}


/******************************************************************************
 * Function:        void USBCBWakeFromSuspend(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The host may put USB peripheral devices in low power
 *                  suspend mode (by "sending" 3+ms of idle).  Once in suspend
 *                  mode, the host may wake the device back up by sending non-
 *                  idle state signalling.
 *                  
 *                  This call back is invoked when a wakeup from USB suspend 
 *                  is detected.
 *
 * Note:            None
 *****************************************************************************/
void USBCBWakeFromSuspend( void )
	{
    // If clock switching or other power savings measures were taken when
    // executing the USBCBSuspend() function, now would be a good time to
    // switch back to normal full power run mode conditions.  The host allows
    // a few milliseconds of wakeup time, after which the device must be 
    // fully back to normal, and capable of receiving and processing USB
    // packets.  In order to do this, the USB module must receive proper
    // clocking (IE: 48MHz clock must be available to SIE for full speed USB
    // operation).
	}

/********************************************************************
 * Function:        void USBCB_SOF_Handler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USB host sends out a SOF packet to full-speed
 *                  devices every 1 ms. This interrupt may be useful
 *                  for isochronous pipes. End designers should
 *                  implement callback routine as necessary.
 *
 * Note:            None
 *******************************************************************/
void USBCB_SOF_Handler(void)
{
    // No need to clear UIRbits.SOFIF to 0 here.
    // Callback caller is already doing that.
}

/*******************************************************************
 * Function:        void USBCBErrorHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The purpose of this callback is mainly for
 *                  debugging during development. Check UEIR to see
 *                  which error causes the interrupt.
 *
 * Note:            None
 *******************************************************************/
void USBCBErrorHandler( void )
	{
    // No need to clear UEIR to 0 here.
    // Callback caller is already doing that.

    // Typically, user firmware does not need to do anything special
    // if a USB error occurs.  For example, if the host sends an OUT
    // packet to your device, but the packet gets corrupted (ex:
    // because of a bad connection, or the user unplugs the
    // USB cable during the transmission) this will typically set
    // one or more USB error interrupt flags.  Nothing specific
    // needs to be done however, since the SIE will automatically
    // send a "NAK" packet to the host.  In response to this, the
    // host will normally retry to send the packet again, and no
    // data loss occurs.  The system will typically recover
    // automatically, without the need for application firmware
    // intervention.
    
    // Nevertheless, this callback function is provided, such as
    // for debugging purposes.
	}


/*******************************************************************
 * Function:        void USBCBCheckOtherReq(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        When SETUP packets arrive from the host, some
 *                  firmware must process the request and respond
 *                  appropriately to fulfill the request.  Some of
 *                  the SETUP packets will be for standard
 *                  USB "chapter 9" (as in, fulfilling chapter 9 of
 *                  the official USB specifications) requests, while
 *                  others may be specific to the USB device class
 *                  that is being implemented.  For example, a HID
 *                  class device needs to be able to respond to
 *                  "GET REPORT" type of requests.  This
 *                  is not a standard USB chapter 9 request, and 
 *                  therefore not handled by usb_device.c.  Instead
 *                  this request should be handled by class specific 
 *                  firmware, such as that contained in usb_function_hid.c.
 *
 * Note:            None
 *******************************************************************/
void USBCBCheckOtherReq( void )
	{
    USBCheckCDCRequest();
	}//end


/*******************************************************************
 * Function:        void USBCBStdSetDscHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USBCBStdSetDscHandler() callback function is
 *                  called when a SETUP, bRequest: SET_DESCRIPTOR request
 *                  arrives.  Typically SET_DESCRIPTOR requests are
 *                  not used in most applications, and it is
 *                  optional to support this type of request.
 *
 * Note:            None
 *******************************************************************/
void USBCBStdSetDscHandler( void )
	{
    // Must claim session ownership if supporting this request
	}//end


/*******************************************************************
 * Function:        void USBCBInitEP(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called when the device becomes
 *                  initialized, which occurs after the host sends a
 *                  SET_CONFIGURATION (wValue not = 0) request.  This 
 *                  callback function should initialize the endpoints 
 *                  for the device's usage according to the current 
 *                  configuration.
 *
 * Note:            None
 *******************************************************************/
void USBCBInitEP( void )
    {
    CDCInitEP();
    }

/********************************************************************
 * Function:        void USBCBSendResume(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USB specifications allow some types of USB
 *                  peripheral devices to wake up a host PC (such
 *                  as if it is in a low power suspend to RAM state).
 *                  This can be a very useful feature in some
 *                  USB applications, such as an Infrared remote
 *                  control	receiver.  If a user presses the "power"
 *                  button on a remote control, it is nice that the
 *                  IR receiver can detect this signalling, and then
 *                  send a USB "command" to the PC to wake up.
 *                  
 *                  The USBCBSendResume() "callback" function is used
 *                  to send this special USB signalling which wakes 
 *                  up the PC.  This function may be called by
 *                  application firmware to wake up the PC.  This
 *                  function should only be called when:
 *                  
 *                  1.  The USB driver used on the host PC supports
 *                      the remote wakeup capability.
 *                  2.  The USB configuration descriptor indicates
 *                      the device is remote wakeup capable in the
 *                      bmAttributes field.
 *                  3.  The USB host PC is currently sleeping,
 *                      and has previously sent your device a SET 
 *                      FEATURE setup packet which "armed" the
 *                      remote wakeup capability.   
 *
 *                  This callback should send a RESUME signal that
 *                  has the period of 1-15ms.
 *
 * Note:            Interrupt vs. Polling
 *                  -Primary clock
 *                  -Secondary clock ***** MAKE NOTES ABOUT THIS *******
 *                   > Can switch to primary first by calling USBCBWakeFromSuspend()
 
 *                  The modifiable section in this routine should be changed
 *                  to meet the application needs. Current implementation
 *                  temporary blocks other functions from executing for a
 *                  period of 1-13 ms depending on the core frequency.
 *
 *                  According to USB 2.0 specification section 7.1.7.7,
 *                  "The remote wakeup device must hold the resume signaling
 *                  for at lest 1 ms but for no more than 15 ms."
 *                  The idea here is to use a delay counter loop, using a
 *                  common value that would work over a wide range of core
 *                  frequencies.
 *                  That value selected is 1800. See table below:
 *                  ==========================================================
 *                  Core Freq(MHz)      MIP         RESUME Signal Period (ms)
 *                  ==========================================================
 *                      48              12          1.05
 *                       4              1           12.6
 *                  ==========================================================
 *                  * These timing could be incorrect when using code
 *                    optimization or extended instruction mode,
 *                    or when having other interrupts enabled.
 *                    Make sure to verify using the MPLAB SIM's Stopwatch
 *                    and verify the actual signal on an oscilloscope.
 *******************************************************************/
void USBCBSendResume( void )
    {
    static WORD delay_count;
    
    USBResumeControl = 1;       // Start RESUME signaling
    
    delay_count = 1800U;        // Set RESUME line for 1-13 ms
    do
    {
        delay_count--;
    }while(delay_count);
    USBResumeControl = 0;
    }


/*******************************************************************
 * Function:        void USBCBEP0DataReceived(void)
 *
 * PreCondition:    ENABLE_EP0_DATA_RECEIVED_CALLBACK must be
 *                  defined already (in usb_config.h)
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called whenever a EP0 data
 *                  packet is received.  This gives the user (and
 *                  thus the various class examples a way to get
 *                  data that is received via the control endpoint.
 *                  This function needs to be used in conjunction
 *                  with the USBCBCheckOtherReq() function since 
 *                  the USBCBCheckOtherReq() function is the apps
 *                  method for getting the initial control transfer
 *                  before the data arrives.
 *
 * Note:            None
 *******************************************************************/
#if defined(ENABLE_EP0_DATA_RECEIVED_CALLBACK)
void USBCBEP0DataReceived( void )
    {
    }
#endif


/*******************************************************************
 * Function:        BOOL USER_USB_CALLBACK_EVENT_HANDLER(
 *                        USB_EVENT event, void *pdata, WORD size)
 *
 * PreCondition:    None
 *
 * Input:           USB_EVENT event - the type of event
 *                  void *pdata - pointer to the event data
 *                  WORD size - size of the event data
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called from the USB stack to
 *                  notify a user application that a USB event
 *                  occured.  This callback is in interrupt context
 *                  when the USB_INTERRUPT option is selected.
 *
 * Note:            None
 *******************************************************************/
BOOL USER_USB_CALLBACK_EVENT_HANDLER( USB_EVENT event, void *pdata, WORD size )
    {
    switch(event)
        {
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        case EVENT_TRANSFER:
            Nop();
            break;
        default:
            break;
        }      
    return TRUE; 
    }

/** EOF main.c *************************************************/
