/*============================================================================*/
/*
SimpleRTOS - Very simple RTOS for Microcontrollers
v2.00 (2014-01-21)
isaacbavaresco@yahoo.com.br
*/
/*============================================================================*/
/*
 Copyright (c) 2007-2014, Isaac Marino Bavaresco
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
     * Neither the name of the author nor the
       names of its contributors may be used to endorse or promote products
       derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ''AS IS'' AND ANY
 EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*============================================================================*/
#if			defined __XC32__
/*============================================================================*/

#include "../SimpleRTOS/SerialPIC32.h"

#define	PLATFORM_ROM
#define	PLATFORM_ClrWdt()
#define	PLATFORM_mainreturn_t	int
#define	PLATFORM_mainreturn(v)	return (v)

#define	APPLICATION_UART		2
#define	APPLICATION_DELAY_1		-1
#define	APPLICATION_DELAY_2		 2
#define	APPLICATION_TICKLENGTH	40000
#define	APPLICATION_STACK_SIZE	256

/*============================================================================*/
#elif		defined __XC16__
/*============================================================================*/

#include "../SimpleRTOS/SerialPIC24dsPIC.h"

#define	PLATFORM_ROM
#define	PLATFORM_ClrWdt()		ClrWdt()
#define	PLATFORM_mainreturn_t	int
#define	PLATFORM_mainreturn(v)	return (v)

#define	APPLICATION_UART		 1
#define	APPLICATION_DELAY_1		-1
#define	APPLICATION_DELAY_2		 2
#define	APPLICATION_TICKLENGTH	40000
#define	APPLICATION_STACK_SIZE	128

/*============================================================================*/
#elif		defined __XC8__ || defined __18CXX
/*============================================================================*/

#include "../SimpleRTOS/SerialPIC18.h"

#define	PLATFORM_ROM			rom
#define	PLATFORM_ClrWdt()		ClrWdt()
#define	PLATFORM_mainreturn_t	void
#define	PLATFORM_mainreturn(v)

#define	APPLICATION_UART		 1
#define	APPLICATION_DELAY_1		11
#define	APPLICATION_DELAY_2		30
#define	APPLICATION_TICKLENGTH	10000
#define	APPLICATION_STACK_SIZE	256

/*============================================================================*/
#elif		defined _MPC_
/*============================================================================*/

#error "Platform not supported."
/*
#include "../SimpleRTOS/SerialPIC18.h"

#define	PLATFORM_ROM			rom
#define	PLATFORM_ClrWdt()		ClrWdt()
#define	PLATFORM_mainreturn_t	void
#define	PLATFORM_mainreturn(v)

#define	APPLICATION_UART		 1
#define	APPLICATION_DELAY_1		11
#define	APPLICATION_DELAY_2		30
#define	APPLICATION_TICKLENGTH	10000
#define	APPLICATION_STACK_SIZE	128
*/

/*============================================================================*/
#else
/*============================================================================*/

#error "Platform not supported."
	
/*============================================================================*/
#endif
/*============================================================================*/

extern unsigned char	Stacks[6][APPLICATION_STACK_SIZE];

void	MainTask	( void* );

/*============================================================================*/
